/*
 *  ******************************************************************************
 *  *
 *  *
 *  * This program and the accompanying materials are made available under the
 *  * terms of the Apache License, Version 2.0 which is available at
 *  * https://www.apache.org/licenses/LICENSE-2.0.
 *  *
 *  *  See the NOTICE file distributed with this work for additional
 *  *  information regarding copyright ownership.
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  * License for the specific language governing permissions and limitations
 *  * under the License.
 *  *
 *  * SPDX-License-Identifier: Apache-2.0
 *  *****************************************************************************
 */

package org.deeplearning4j.nn.conf.layers;

import lombok.*;
import lombok.experimental.SuperBuilder;
import org.nd4j.linalg.activations.Activation;
import org.nd4j.linalg.activations.IActivation;
import org.nd4j.linalg.activations.impl.ActivationSigmoid;
import org.nd4j.linalg.activations.impl.ActivationTanH;


@ToString(callSuper = true)
@EqualsAndHashCode(callSuper = true)
@SuperBuilder
public abstract class AbstractLSTM extends BaseRecurrentLayer {

  /**
   * Set forget gate bias initalizations. Values in range 1-5 can potentially help with learning or
   * longer-term dependencies.
   */
  @lombok.Builder.Default @Getter protected double forgetGateBiasInit = 1.0;
  /**
   * When using CuDNN and an error is encountered, should fallback to the non-CuDNN implementatation
   * be allowed? If set to false, an exception in CuDNN will be propagated back to the user. If
   * false, the built-in (non-CuDNN) implementation for LSTM/GravesLSTM will be used
   */
  @lombok.Builder.Default @Getter protected boolean helperAllowFallback = true;
  /**
   * Activation function for the LSTM gates. Note: This should be bounded to range 0-1: sigmoid or
   * hard sigmoid, for example
   */
  @lombok.Builder.Default @Getter @Setter private IActivation activation = new ActivationSigmoid();

  /**
   * Activation function for the LSTM gates. Note: This should be bounded to range 0-1: sigmoid or
   * hard sigmoid, for example
   */
  @lombok.Builder.Default @Getter @Setter
  private IActivation gateActivationFunction = new ActivationTanH();
}
