/*
 *  ******************************************************************************
 *  *
 *  *
 *  * This program and the accompanying materials are made available under the
 *  * terms of the Apache License, Version 2.0 which is available at
 *  * https://www.apache.org/licenses/LICENSE-2.0.
 *  *
 *  *  See the NOTICE file distributed with this work for additional
 *  *  information regarding copyright ownership.
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  * License for the specific language governing permissions and limitations
 *  * under the License.
 *  *
 *  * SPDX-License-Identifier: Apache-2.0
 *  *****************************************************************************
 */

package org.deeplearning4j.nn.conf.layers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import lombok.*;
import lombok.experimental.SuperBuilder;
import org.deeplearning4j.nn.api.Layer;
import org.deeplearning4j.nn.api.ParamInitializer;
import org.deeplearning4j.nn.api.layers.LayerConstraint;
import org.deeplearning4j.nn.conf.NeuralNetConfiguration;
import org.deeplearning4j.nn.conf.inputs.InputType;
import org.deeplearning4j.nn.conf.memory.LayerMemoryReport;
import org.deeplearning4j.nn.layers.recurrent.LSTMHelpers;
import org.deeplearning4j.nn.params.GravesLSTMParamInitializer;
import org.deeplearning4j.optimize.api.TrainingListener;
import org.nd4j.linalg.activations.IActivation;
import org.nd4j.linalg.activations.impl.ActivationSigmoid;
import org.nd4j.linalg.api.buffer.DataType;
import org.nd4j.linalg.api.ndarray.INDArray;

@Deprecated
@ToString(callSuper = true)
@EqualsAndHashCode(callSuper = true)
@SuperBuilder(buildMethodName = "initBuild")
public class GravesLSTM extends AbstractLSTM {

  private double forgetGateBiasInit;
  @Builder.Default @Getter private IActivation gateActivationFunction = new ActivationSigmoid();

  @Override
  protected void initializeConstraints() {
    super.initializeConstraints();
    if (getRecurrentConstraints() != null) {
      if (constraints == null) {
        constraints = new ArrayList<>();
      }
      for (LayerConstraint c : getRecurrentConstraints()) {
        LayerConstraint c2 = c.clone();
        c2.setParams(Collections.singleton(GravesLSTMParamInitializer.RECURRENT_WEIGHT_KEY));
        constraints.add(c2);
      }
    }
  }

  @Override
  public Layer instantiate(
      NeuralNetConfiguration conf,
      Collection<TrainingListener> trainingListeners,
      int layerIndex,
      INDArray layerParamsView,
      boolean initializeParams,
      DataType networkDataType) {
    LayerValidation.assertNInNOutSet("GravesLSTM", getName(), layerIndex, getNIn(), getNOut());

    LayerConfiguration lconf = conf.getFlattenedLayerConfigurations().get(layerIndex);
    lconf.setNetConfiguration(conf);
    runInheritance();

    org.deeplearning4j.nn.layers.recurrent.GravesLSTM ret =
        new org.deeplearning4j.nn.layers.recurrent.GravesLSTM(lconf, networkDataType);

    ret.addTrainingListeners(trainingListeners);
    ret.setIndex(layerIndex);
    ret.setParamsViewArray(layerParamsView);
    Map<String, INDArray> paramTable = initializer().init(this, layerParamsView, initializeParams);
    ret.setParamTable(paramTable);
    ret.setLayerConfiguration(lconf);
    return ret;
  }

  @Override
  public ParamInitializer initializer() {
    return GravesLSTMParamInitializer.getInstance();
  }

  @Override
  public LayerMemoryReport getMemoryReport(InputType inputType) {
    // TODO - CuDNN etc
    return LSTMHelpers.getMemoryReport(this, inputType);
  }

  public abstract static class GravesLSTMBuilder<
          C extends GravesLSTM, B extends GravesLSTMBuilder<C, B>>
      extends AbstractLSTMBuilder<C, B> {
    public C build() {
      C l = initBuild();
      l.initializeConstraints();
      return l;
    }
  }
}
